mod table_directory;
mod character_map;
mod search;

use std::{fs::File, io::{BufReader, Read}};

use bincode::Options;
use character_map::CharacterMap;
use log::{debug, info};
use table_directory::TableDirectory;

fn deserialize<R: Read, T: serde::de::DeserializeOwned>(reader: R) -> bincode::Result<T> {
	bincode::options().with_big_endian().with_fixint_encoding().deserialize_from::<R, T>(reader)
}

#[derive(Debug)]
pub struct Font {
	data_source: BufReader<File>,
	table_directory: TableDirectory,
	character_map: CharacterMap
}
	
impl Font {
	pub fn new(file: File) -> bincode::Result<Self> {
		let mut buf_reader = BufReader::new(file.try_clone()?);
		let table_directory = TableDirectory::new(&mut buf_reader)?;

		let Some(cmap) = table_directory.get_table("cmap") else {
			return Err(Box::new(bincode::ErrorKind::Custom("Missing table 'cmap'".into())));
		};

		let character_map = CharacterMap::new(file.try_clone()?, cmap)?;
		debug!("{character_map:#?}");

		Ok(Font {
			data_source: buf_reader,
			table_directory,
			character_map
		})
	}

	pub fn get_data_for(&mut self, glyph: char) -> Result<(), &'static str> {
		let Some(table) = self.table_directory.get_table("glyf") else {
			return Err("Failed to get table. It doesn't exist");
		};

		let Some(glyph_id) = self.character_map.get_char(glyph) else {
			return Err("Failed to get glyph id for char");
		};

		info!("glyph id for char '{glyph}' is {glyph_id}");

		Ok(())
	}
}

